@@ -20,13 +20,13 @@ class ColumnLoader(LoaderStrategy):
         self.columns = self.parent_property.columns
         self._should_log_debug = logging.is_debug_enabled(self.logger)
         
-    def setup_query(self, context, eagertable=None, **kwargs):
+    def setup_query(self, context, eagertable=None, parentclauses=None, **kwargs):
         for c in self.columns:
-            if eagertable is not None:
-                context.statement.append_column(eagertable.corresponding_column(c))
+            if parentclauses is not None:
+                context.statement.append_column(parentclauses.aliased_column(c))
             else:
                 context.statement.append_column(c)
-
+        
     def init_class_attribute(self):
         self.logger.info("register managed attribute %s on class %s" % (self.key, self.parent.class_.__name__))
         coltype = self.columns[0].type
@@ -161,11 +161,15 @@ NoLoader.logger = logging.class_logger(NoLoader)
 class LazyLoader(AbstractRelationLoader):
     def init(self):
         super(LazyLoader, self).init()
-        (self.lazywhere, self.lazybinds, self.lazyreverse) = self._create_lazy_clause(self.polymorphic_primaryjoin, self.polymorphic_secondaryjoin, self.remote_side)
+        (self.lazywhere, self.lazybinds, self.lazyreverse) = self._create_lazy_clause(self)
+        
+        self.logger.info(str(self.parent_property) + " lazy loading clause " + str(self.lazywhere))
 
         # determine if our "lazywhere" clause is the same as the mapper's
         # get() clause.  then we can just use mapper.get()
         self.use_get = not self.uselist and query.Query(self.mapper)._get_clause.compare(self.lazywhere)
+        if self.use_get:
+            self.logger.info(str(self.parent_property) + " will use query.get() to optimize instance loads")
 
     def init_class_attribute(self):
         self._register_attribute(self.parent.class_, callable_=lambda i: self.setup_loader(i))
@@ -246,12 +250,17 @@ class LazyLoader(AbstractRelationLoader):
                 # to load data into it.
                 sessionlib.attribute_manager.reset_instance_attribute(instance, self.key)
 
-    def _create_lazy_clause(self, primaryjoin, secondaryjoin, remote_side):
+    def _create_lazy_clause(cls, prop, reverse_direction=False):
+        (primaryjoin, secondaryjoin, remote_side) = (prop.polymorphic_primaryjoin, prop.polymorphic_secondaryjoin, prop.remote_side)
+        
         binds = {}
         reverse = {}
 
         def should_bind(targetcol, othercol):
-            return othercol in remote_side
+            if reverse_direction and not secondaryjoin:
+                return targetcol in remote_side
+            else:
+                return othercol in remote_side
 
         def find_column_in_expr(expr):
             if not isinstance(expr, sql.ColumnElement):
@@ -263,14 +272,8 @@ class LazyLoader(AbstractRelationLoader):
             FindColumnInColumnClause().traverse(expr)
             return len(columns) and columns[0] or None
         
-        def col_in_collection(column, collection):
-            for c in collection:
-                if column.shares_lineage(c):
-                    return True
-            else:
-                return False
-                
         def bind_label():
+            # TODO: make this generation deterministic
             return "lazy_" + hex(random.randint(0, 65535))[2:]
 
         def visit_binary(binary):
@@ -294,15 +297,19 @@ class LazyLoader(AbstractRelationLoader):
 
         lazywhere = primaryjoin.copy_container()
         li = mapperutil.BinaryVisitor(visit_binary)
-        li.traverse(lazywhere)
+        
+        if not secondaryjoin or not reverse_direction:
+            li.traverse(lazywhere)
         
         if secondaryjoin is not None:
             secondaryjoin = secondaryjoin.copy_container()
+            if reverse_direction:
+                li.traverse(secondaryjoin)
             lazywhere = sql.and_(lazywhere, secondaryjoin)
- 
-        LazyLoader.logger.info(str(self.parent_property) + " lazy loading clause " + str(lazywhere))
-        return (lazywhere, binds, reverse)
 
+        return (lazywhere, binds, reverse)
+    _create_lazy_clause = classmethod(_create_lazy_clause)
+    
 LazyLoader.logger = logging.class_logger(LazyLoader)
 
 
@@ -312,8 +319,13 @@ class EagerLoader(AbstractRelationLoader):
     def init(self):
         super(EagerLoader, self).init()
         if self.parent.isa(self.mapper):
-            raise exceptions.ArgumentError("Error creating eager relationship '%s' on parent class '%s' to child class '%s': Cant use eager loading on a self referential relationship." % (self.key, repr(self.parent.class_), repr(self.mapper.class_)))
-        self.parent._eager_loaders.add(self.parent_property)
+            raise exceptions.ArgumentError(
+                "Error creating eager relationship '%s' on parent class '%s' "
+                "to child class '%s': Cant use eager loading on a self "
+                "referential relationship." %
+                (self.key, repr(self.parent.class_), repr(self.mapper.class_)))
+        if self.is_default:
+            self.parent._eager_loaders.add(self.parent_property)
 
         self.clauses = {}
         self.clauses_by_lead_mapper = {}
@@ -352,43 +364,65 @@ class EagerLoader(AbstractRelationLoader):
         """
         
         def __init__(self, eagerloader, parentclauses=None):
+            self.id = (parentclauses is not None and (parentclauses.id + "/") or '') + str(eagerloader.parent_property)
             self.parent = eagerloader
             self.target = eagerloader.select_table
-            self.eagertarget = eagerloader.select_table.alias()
-            
+            self.eagertarget = eagerloader.select_table.alias(self._aliashash("/target"))
+            self.extra_cols = {}
+
             if eagerloader.secondary:
-                self.eagersecondary = eagerloader.secondary.alias()
-                self.aliasizer = sql_util.Aliasizer(eagerloader.target, eagerloader.secondary, aliases={
-                        eagerloader.target:self.eagertarget,
-                        eagerloader.secondary:self.eagersecondary
-                        })
+                self.eagersecondary = eagerloader.secondary.alias(self._aliashash("/secondary"))
+                if parentclauses is not None:
+                    aliasizer = sql_util.ClauseAdapter(self.eagertarget).\
+                            chain(sql_util.ClauseAdapter(self.eagersecondary)).\
+                            chain(sql_util.ClauseAdapter(parentclauses.eagertarget))
+                else:
+                    aliasizer = sql_util.ClauseAdapter(self.eagertarget).\
+                        chain(sql_util.ClauseAdapter(self.eagersecondary))
                 self.eagersecondaryjoin = eagerloader.polymorphic_secondaryjoin.copy_container()
-                self.aliasizer.traverse(self.eagersecondaryjoin)
+                aliasizer.traverse(self.eagersecondaryjoin)
                 self.eagerprimary = eagerloader.polymorphic_primaryjoin.copy_container()
-                self.aliasizer.traverse(self.eagerprimary)
+                aliasizer.traverse(self.eagerprimary)
             else:
                 self.eagerprimary = eagerloader.polymorphic_primaryjoin.copy_container()
-                self.aliasizer = sql_util.Aliasizer(self.target, aliases={self.target:self.eagertarget})
-                self.aliasizer.traverse(self.eagerprimary)
-
-            if parentclauses is not None:
-                parentclauses.aliasizer.traverse(self.eagerprimary)
+                if parentclauses is not None: 
+                    aliasizer = sql_util.ClauseAdapter(self.eagertarget)
+                    aliasizer.chain(sql_util.ClauseAdapter(parentclauses.eagertarget, exclude=eagerloader.parent_property.remote_side))
+                else:
+                    aliasizer = sql_util.ClauseAdapter(self.eagertarget)
+                aliasizer.traverse(self.eagerprimary)
 
             if eagerloader.order_by:
-                self.eager_order_by = self._aliasize_orderby(eagerloader.order_by)
+                self.eager_order_by = sql_util.ClauseAdapter(self.eagertarget).copy_and_process(util.to_list(eagerloader.order_by))
             else:
                 self.eager_order_by = None
 
             self._row_decorator = self._create_decorator_row()
+        
+        def aliased_column(self, column):
+            """return the aliased version of the given column, creating a new label for it if not already
+            present in this AliasedClauses eagertable."""
 
-        def _aliasize_orderby(self, orderby, copy=True):
-            if copy:
-                return self.aliasizer.copy_and_process(util.to_list(orderby))
-            else:
-                orderby = util.to_list(orderby)
-                self.aliasizer.process_list(orderby)
-                return orderby
+            conv = self.eagertarget.corresponding_column(column, raiseerr=False)
+            if conv:
+                return conv
 
+            if column in self.extra_cols:
+                return self.extra_cols[column]
+            
+            aliased_column = column.copy_container()
+            sql_util.ClauseAdapter(self.eagertarget).traverse(aliased_column)
+            alias = self._aliashash(column.name)
+            aliased_column = aliased_column.label(alias)
+            self._row_decorator.map[column] = alias
+            self.extra_cols[column] = aliased_column
+            return aliased_column
+            
+        def _aliashash(self, extra):
+            """return a deterministic 4 digit hash value for this AliasedClause's id + extra."""
+            # use the first 4 digits of an MD5 hash
+            return "anon_" + util.hash(self.id + extra)[0:4]
+            
         def _create_decorator_row(self):
             class EagerRowAdapter(object):
                 def __init__(self, row):
@@ -407,6 +441,7 @@ class EagerLoader(AbstractRelationLoader):
                 map[parent] = c
                 map[parent._label] = c
                 map[parent.name] = c
+            EagerRowAdapter.map = map
             return EagerRowAdapter
 
         def _decorate_row(self, row):
@@ -435,22 +470,23 @@ class EagerLoader(AbstractRelationLoader):
         
         if hasattr(statement, '_outerjoin'):
             towrap = statement._outerjoin
-        elif isinstance(localparent.mapped_table, schema.Table):
-            # if the mapper is against a plain Table, look in the from_obj of the select statement
-            # to join against whats already there.
-            for (fromclause, finder) in [(x, sql_util.TableFinder(x)) for x in statement.froms]:
-                # dont join against an Alias'ed Select.  we are really looking either for the 
-                # table itself or a Join that contains the table.  this logic still might need
-                # adjustments for scenarios not thought of yet.
-                if not isinstance(fromclause, sql.Alias) and localparent.mapped_table in finder:
+        elif isinstance(localparent.mapped_table, sql.Join):
+            towrap = localparent.mapped_table
+        else:
+            # look for the mapper's selectable expressed within the current "from" criterion.
+            # this will locate the selectable inside of any containers it may be a part of (such
+            # as a join).  if its inside of a join, we want to outer join on that join, not the 
+            # selectable.
+            for fromclause in statement.froms:
+                if fromclause is localparent.mapped_table:
                     towrap = fromclause
                     break
+                elif isinstance(fromclause, sql.Join):
+                    if localparent.mapped_table in sql_util.TableFinder(fromclause, include_aliases=True):
+                        towrap = fromclause
+                        break
             else:
-                raise exceptions.InvalidRequestError("EagerLoader cannot locate a clause with which to outer join to, in query '%s' %s" % (str(statement), self.localparent.mapped_table))
-        else:
-            # if the mapper is against a select statement or something, we cant handle that at the
-            # same time as a custom FROM clause right now.
-            towrap = localparent.mapped_table
+                raise exceptions.InvalidRequestError("EagerLoader cannot locate a clause with which to outer join to, in query '%s' %s" % (str(statement), localparent.mapped_table))
         
         try:
             clauses = self.clauses[parentclauses]
