@@ -10,6 +10,8 @@ except ImportError:
     import dummy_thread as thread
     import dummy_threading as threading
 
+import md5
+
 import __builtin__
 
 try:
@@ -56,6 +58,12 @@ def flatten_iterator(x):
         else:
             yield elem
 
+def hash(string):
+    """return an md5 hash of the given string."""
+    h = md5.new()
+    h.update(string)
+    return h.hexdigest()
+    
 
 class ArgSingleton(type):
     instances = {}
@@ -86,6 +94,39 @@ def get_cls_kwargs(cls):
                     kw.append(vn)
     return kw
 
+def get_func_kwargs(func):
+    """Return the full set of legal kwargs for the given `func`."""
+    return [vn for vn in func.func_code.co_varnames]
+
+def coerce_kw_type(kw, key, type_, flexi_bool=True):
+    """If 'key' is present in dict 'kw', coerce its value to type 'type_' if
+    necessary.  If 'flexi_bool' is True, the string '0' is considered false
+    when coercing to boolean.
+    """
+
+    if key in kw and type(kw[key]) is not type_ and kw[key] is not None:
+        if type_ is bool and flexi_bool and kw[key] == '0':
+            kw[key] = False
+        else:
+            kw[key] = type_(kw[key])
+
+def duck_type_collection(col, default=None):
+    """Given an instance or class, guess if it is or is acting as one of
+    the basic collection types: list, set and dict.  If the __emulates__
+    property is present, return that preferentially.
+    """
+    
+    if hasattr(col, '__emulates__'):
+        return getattr(col, '__emulates__')
+    elif hasattr(col, 'append'):
+        return list
+    elif hasattr(col, 'add'):
+        return Set
+    elif hasattr(col, 'set'):
+        return dict
+    else:
+        return default
+    
 class SimpleProperty(object):
     """A *default* property accessor."""
 
@@ -104,6 +145,24 @@ class SimpleProperty(object):
         else:
             return getattr(obj, self.key)
 
+class NotImplProperty(object):
+  """a property that raises ``NotImplementedError``."""
+  
+  def __init__(self, doc):
+      self.__doc__ = doc
+      
+  def __set__(self, obj, value):
+      raise NotImplementedError()
+
+  def __delete__(self, obj):
+      raise NotImplementedError()
+
+  def __get__(self, obj, owner):
+      if obj is None:
+          return self
+      else:
+          raise NotImplementedError()
+  
 class OrderedProperties(object):
     """An object that maintains the order in which attributes are set upon it.
 
@@ -290,6 +349,9 @@ class OrderedSet(Set):
       super(OrderedSet, self).clear()
       self._list=[]
 
+    def __getitem__(self, key):
+        return self._list[key]
+
     def __iter__(self):
         return iter(self._list)
 
