@@ -513,82 +513,6 @@ class PassiveDeletesTest(UnitOfWorkTest):
         
 
         
-class PrivateAttrTest(UnitOfWorkTest):
-    """tests various things to do with private=True mappers"""
-    def setUpAll(self):
-        UnitOfWorkTest.setUpAll(self)
-        global a_table, b_table
-        a_table = Table('a',testbase.db,
-            Column('a_id', Integer, Sequence('next_a_id'), primary_key=True),
-            Column('data', String(10)),
-            )
-    
-        b_table = Table('b',testbase.db,
-            Column('b_id',Integer,Sequence('next_b_id'),primary_key=True),
-            Column('a_id',Integer,ForeignKey('a.a_id')),
-            Column('data',String(10)))
-        a_table.create()
-        b_table.create()
-    def tearDownAll(self):
-        b_table.drop()
-        a_table.drop()
-        UnitOfWorkTest.tearDownAll(self)
-    
-    def testsinglecommit(self):
-        """tests that a commit of a single object deletes private relationships"""
-        class A(object):pass
-        class B(object):pass
-    
-        assign_mapper(B,b_table)
-        assign_mapper(A,a_table,properties= {'bs' : relation(B.mapper,private=True)})
-    
-        # create some objects
-        a = A(data='a1')
-        a.bs = []
-    
-        # add a 'B' instance
-        b1 = B(data='1111')
-        a.bs.append(b1)
-    
-        # add another one
-        b2 = B(data='2222')
-        a.bs.append(b2)
-    
-        # inserts both A and Bs
-        ctx.current.flush([a])
-    
-        ctx.current.delete(a)
-        print ctx.current.deleted
-        ctx.current.flush([a])
-#        ctx.current.flush()
-        
-        assert b_table.count().scalar() == 0
-
-    def testswitchparent(self):
-        """tests that you can switch the parent of an object in a backref scenario"""
-        class A(object):pass
-        class B(object):pass
-    
-        assign_mapper(B,b_table)
-        assign_mapper(A,a_table,properties= {
-            'bs' : relation (B.mapper,private=True, backref='a')}
-        )
-        a1 = A(data='testa1')
-        a2 = A(data='testa2')
-        b = B(data='testb')
-        b.a = a1
-        ctx.current.flush()
-        ctx.current.clear()
-        sess = ctx.current
-        a1 = Query(A).get(a1.a_id)
-        a2 = Query(A).get(a2.a_id)
-        assert a1.bs[0].a is a1
-        b = a1.bs[0]
-        b.a = a2
-        assert b not in sess.deleted
-        ctx.current.flush()
-        assert b in sess.identity_map.values()
-                
 class DefaultTest(UnitOfWorkTest):
     """tests that when saving objects whose table contains DefaultGenerators, either python-side, preexec or database-side,
     the newly saved instances receive all the default values either through a post-fetch or getting the pre-exec'ed 
